<?php

namespace Drupal\field_states\Form;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\field_states\StatesTransitionService;

/**
 * Transition form.
 */
class StateTransitionForm extends FormBase implements StateTransitionFormInterface {

  /**
   * The entity.
   *
   * @var \Drupal\Core\Entity\ContentEntityInterface
   */
  protected $entity;

  /**
   * The state field name.
   *
   * @var string
   */
  protected $fieldName;

  /**
   * The state transition service.
   *
   * @var \Drupal\field_states\StatesTransitionService
   */
  protected StatesTransitionService $stateService;

  /**
   * Constructs a transition form object.
   *
   * @param \Drupal\field_states\StatesTransitionService $state_service
   *   The entity type manager.
   */
  public function __construct(StatesTransitionService $state_service) {
    $this->stateService = $state_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create($container) {
    return new static(
      $container->get('field_states.transitions'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getEntity() {
    return $this->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function setEntity(ContentEntityInterface $entity) {
    $this->entity = $entity;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldName() {
    return $this->fieldName;
  }

  /**
   * {@inheritdoc}
   */
  public function setFieldName($field_name) {
    $this->fieldName = $field_name;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getBaseFormId() {
    return 'state_machine_transition_form';
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    $entity = $this->getEntity();
    if (!$entity) {
      throw new \RuntimeException('No entity provided to StateTransitionForm.');
    }
    // Example ID: "state_machine_transition_form_commerce_order_state_1".
    $form_id = $this->getBaseFormId();
    $form_id .= '_' . $entity->getEntityTypeId() . '_' . $this->fieldName;
    $form_id .= '_' . $entity->id();

    return $form_id;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\state_machine\Plugin\Field\FieldType\StateItemInterface $state_item */
    $state_item = $this->entity->get($this->fieldName)->first();
    $state = !empty($state_item) ? $state_item->get('value')->getString() : '';
    $form['actions'] = [
      '#type' => 'container',
    ];
    $transitions = $this->stateService
      ->setTransitionByField($this->entity, $this->fieldName)
      ->getPossibleTransitions($state);
    foreach ($transitions->getTransitions() as $transition_id => $transition) {
      if (empty($transition['id'])) {
        $transition['id'] = $transition_id;
      }
      $allowed = $transitions->isTransitionAllowed($transition, $this->entity);
      if (!$allowed) {
        continue;
      }
      $form['actions'][$transition_id] = [
        '#type' => 'submit',
        '#value' => $transition['label'],
        '#submit' => ['::submitForm'],
        '#transition' => $transition,
      ];
      if (!empty($transition['class'])) {
        $class = is_array($transition['class']) ? $transition['class'] : explode(' ', $transition['class']);
        $form['actions'][$transition_id]['#attributes']['class'] = $class;
      }
    }
    if (empty($transitions)) {
      $form['#empty_transition'] = TRUE;
      $form['#state'] = $state;
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $transition = $triggering_element['#transition'];
    $transitions = $this->stateService->setTransition($triggering_element['#transition']);
    $this->entity->get($this->fieldName)->setValue(['value' => $transition['to']]);
    $transitions->applyTransition($this->entity);
    // Save log.
    $properties = [
      'targetEntityType' => $this->getEntity()->getEntityTypeId(),
      'bundle' => $this->getEntity()->bundle(),
    ];
    $currentUser = \Drupal::currentUser();
    $formatter = \Drupal::service('date.formatter');
    $view_displays = \Drupal::entityTypeManager()
      ->getStorage('entity_view_display')
      ->loadByProperties($properties);
    $view_display = current($view_displays);
    $component = $view_display->getComponent($this->fieldName);
    $fieldSettings = $component['settings'] ?? ['history' => FALSE];
    if (!empty($history = $fieldSettings['history'])) {
      $field_instance = FieldConfig::loadByName($properties['targetEntityType'], $properties['bundle'], $history);
      $fieldType = $field_instance->getFieldStorageDefinition()->getType();

      $timestamp = strtotime('now');
      $log = [
        'date' => $formatter->format($timestamp),
        'from' => $transition["state"],
        'to' => $transition["to"],
        'user' => $currentUser->getDisplayName(),
      ];
      $to = ' → ';
      switch ($fieldType) {
        case 'json':
          $value = $this->entity->get($history)->value;
          $json = json_decode($value ?: '[]');
          $json[] = $log;
          $this->entity->set($history, json_encode($json));
          break;

        case 'string':
          $this->entity->$history->appendItem(
            $log['date'] . ' State: ' . $log['from'] . $to . $log['to'] . ' by: ' . $log['user']
          );
          break;

        case 'double_field':
          $this->entity->$history->appendItem([
            'first' => date('Y-m-d\TH:i:s'),
            'second' => $log['from'] . $to . $log['to'] . ' - ' . $log['user'],
          ]);
          break;

        case 'triples_field':
          $this->entity->$history->appendItem([
            'first' => date('Y-m-d\TH:i:s'),
            'second' => implode($to, [$log['from'], $log['to']]),
            'third' => $log['user'],
          ]);
          break;

      }
    }
    if (!empty($this->entity) && method_exists($this->entity, 'save')) {
      $this->entity->save();
    }
  }

}
