<?php

namespace Drupal\field_states\Plugin\Field\FieldType;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Serialization\Yaml;
use Drupal\Core\Url;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\options\Plugin\Field\FieldType\ListStringItem;

/**
 * Defines the 'list_states' field type.
 *
 * @FieldType(
 *   id = "list_states",
 *   label = @Translation("States transitions"),
 *   category = @Translation("General"),
 *   default_widget = "states_select",
 *   default_formatter = "state_transition",
 *   cardinality = 1,
 * )
 */
final class ListStatesItem extends ListStringItem {

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings(): array {
    $settings = ['workflows' => ''];
    return $settings + parent::defaultFieldSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state): array {
    $element = [];
    $stateDefault = $this->getValue();
    $workflow = $this->getWorkflow();
    $transitions = $this->getTransitions($workflow);
    $stateDiagram = ["stateDiagram-v2"];
    $userInput = $form_state->getUserInput();
    $allowed_values = $this->getSetting('allowed_values');
    $op = $userInput['op'] ?? FALSE;
    if (!empty($op) && $op == 'Save settings') {
      $allowedAdd = FALSE;
      if (!empty($states = $workflow["states"])) {
        foreach ($states as $state => $stateValue) {
          if (empty($allowed_values[$state])) {
            $allowed_values[$state] = $stateValue['label'];
            $allowedAdd = TRUE;
          }
        }
      }
      if (!empty($transitions)) {
        foreach ($transitions as $transition) {
          if (empty($allowed_values[$transition['to']])) {
            $allowed_values[$transition['to']] = $transition['to'];
            $allowedAdd = TRUE;
          }
        }
      }
      if ($allowedAdd) {
        $fieldName = $this->getFieldDefinition()->getName();
        $entityType = $this->getFieldDefinition()->getTargetEntityTypeId();
        $fieldStorage = FieldStorageConfig::loadByName($entityType, $fieldName);
        $fieldStorage->setSetting('allowed_values', $allowed_values);
        $fieldStorage->save();
      }
    }
    foreach ($allowed_values as $state => $stateName) {
      $state = str_replace('-', '_', HTML::getId($state));
      $stateName = str_replace(':', '', $stateName);
      $stateDiagram[] = "$state: $stateName";
    }
    if (!empty($stateDefault['value'])) {
      $stateDefault = $stateDefault['value'];
    }
    else {
      $stateDefault = '[*]';
    }
    if (!empty($transitions)) {
      foreach ($transitions as $transition) {
        foreach ($transition['from'] as $state) {
          if (empty($state)) {
            $state = $stateDefault;
          }
          else {
            $state = str_replace('-', '_', HTML::getId($state));
          }
          $to = !empty($transition['to']) ? HTML::getId($transition['to']) : '[*]';
          $query = trim($state) . ' --> ' . str_replace('-', '_', $to);
          if (!empty($transition['label'])) {
            $query .= ':' . $transition['label'];
          }
          $stateDiagram[] = $query;
        }
      }
    }
    $element['workflows'] = [
      '#title' => $this->t('Workflow'),
      '#type' => 'textarea',
      '#title_display' => 'invisible',
      '#default_value' => is_array($workflow) ? Yaml::encode($workflow) : $workflow,
      '#attributes' => [
        'data-yaml-editor' => 'true',
        'data-editor' => 'settings-workflow',
        'class' => ['js-hide', 'yaml-editor'],
      ],
    ];
    $element['editor'] = [
      '#type' => 'details',
      '#title' => $this->t('Workflow'),
      '#open' => TRUE,
      '#description' => $this->t('Define workflow and transitions'),
      '#value' => [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['layout-row', 'clearfix', 'row'],
        ],
        'workflow' => [
          '#type' => 'container',
          '#description' => $this->t('Define your workflow'),
          '#attributes' => [
            'id' => 'settings-workflow',
            'class' => ['layout-column', 'layout-column--half', 'col-5'],
          ],
        ],
        'state-machine' => [
          '#type' => 'html_tag',
          '#tag' => 'div',
          '#attributes' => [
            'id' => 'stateDiagram',
            'class' => [
              'mermaid',
              'layout-column',
              'layout-column--half',
              'col-7',
            ],
          ],
          '#value' => implode("\n", $stateDiagram),
        ],
        'help' => [
          '#type' => 'link',
          '#url' => Url::fromRoute('help.page', ['name' => 'field_states']),
          '#title' => $this->t('List id: guard, workflow, action'),
          '#attributes' => [
            'class' => ['use-ajax'],
            'data-dialog-type' => 'dialog',
            'data-dialog-options' => Json::encode([
              'dialogClass' => 'token-tree-dialog',
              'width' => 650,
              'height' => 500,
              'draggable' => TRUE,
              'autoResize' => FALSE,
              'position' => [
                'my' => 'right bottom',
                'at' => 'right-10 bottom-10',
              ],
            ]),
          ],
        ],
      ],
    ];
    $element['#attached']['library'][] = 'field_states/field_states';
    return $element;
  }

  /**
   * Get transitions.
   *
   * {@inheritdoc}
   */
  public function getTransitions($workflow = []) {
    if (empty($workflow)) {
      $workflow = $this->getWorkflow();
    }
    return $workflow['transitions'] ?? [];
  }

  /**
   * Get workflow.
   *
   * {@inheritdoc}
   */
  public function getWorkflow() {
    $workflow = trim($this->getSetting('workflows') ?? '');
    if (empty($workflow)) {
      $workflow = $this->getWorkflowDefault();
    }
    if (is_string($workflow)) {
      $workflow = Yaml::decode($workflow);
    }
    return $workflow;
  }

  /**
   * Get transitions.
   *
   * {@inheritdoc}
   */
  public function getWorkflowDefault() {
    $states = $this->getSetting('allowed_values');
    $from = '';
    $transitions = [];
    foreach ($states as $state => $stateLabel) {
      $transitionName = str_replace(' ', '_', $from . '_' . $state);
      $transitions[$transitionName] = [
        'label' => implode(' → ', [$from, $state]),
        'from' => [$from],
        'to' => $state,
        'id' => $transitionName,
      ];
      $from = $state;
      $states[$state] = [
        'label' => $stateLabel,
      ];
    }
    return [
      'states' => $states,
      'transitions' => $transitions,
    ];
  }

}
