<?php

namespace Drupal\field_states;

use Drupal\Core\Action\ActionManager;
use Drupal\Core\Serialization\Yaml;
use Drupal\Core\Session\AccountProxyInterface;

/**
 * Guard plugin manager.
 */
class StatesTransitionService implements StatesTransitionInterface {

  /**
   * The transitions.
   *
   * @var array
   */
  protected array $transitions = [];

  /**
   * The transition.
   *
   * @var array|mixed
   */
  protected mixed $transition = NULL;

  /**
   * Constructs a States Transitions Service object.
   */
  public function __construct(
    protected GuardPluginManager $guard,
    protected WorkflowPluginManager $workflow,
    protected ActionManager $action,
    protected AccountProxyInterface $currentUser,
  ) {
  }

  /**
   * Set transitions form field settings.
   *
   * @param mixed|array $transitions
   *   Transition array.
   *
   * @return $this|mixed
   *   Return this.
   */
  public function setTransitions($transitions) {
    $this->transitions = $transitions;
    return $this;
  }

  /**
   * Set transition to execute.
   *
   * @param mixed|array $transition
   *   Transition array.
   *
   * @return $this
   *   Return this.
   */
  public function setTransition($transition) {
    $this->transition = $transition;
    return $this;
  }

  /**
   * Get transition after method transition.
   *
   * @return mixed|array
   *   Array of transition.
   */
  public function getTransition() {
    return $this->transition;
  }

  /**
   * Get transitions after method getPossibleTransitions.
   *
   * @return array
   *   Array of transition.
   */
  public function getTransitions() {
    return $this->transitions;
  }

  /**
   * Get transition if we know from a state and destination state.
   *
   * @param string $fromState
   *   From state.
   * @param string $toState
   *   Destination state.
   *
   * @return $this|mixed
   *   Return this.
   */
  public function transition($fromState, $toState) {
    foreach ($this->transitions as $transition) {
      if ($transition['to'] === $toState && in_array($fromState, $transition['from'])) {
        $this->setTransition($transition);
        break;
      }
    }
    return $this;
  }

  /**
   * Run transitions.
   *
   * @param object|mixed $entity
   *   Entity object.
   *
   * @return $this|mixed
   *   Return this or null.
   */
  public function applyTransition($entity = NULL): mixed {
    if (!empty($this->transition) && $this->isTransitionAllowed($this->transition, $entity)) {
      if (!empty($this->transition['workflow'])) {
        $workflows = is_array($this->transition['workflow']) ? $this->transition['workflow'] : [$this->transition['workflow']];
        foreach ($workflows as $workflowId) {
          $plugin = $this->workflow->createInstance($workflowId);
          if (!empty($plugin) && method_exists($plugin, 'action')) {
            $plugin->action($this->transition, ['transitions' => $this->transitions], $entity);
          }
        }
      }

      if (!empty($transition['action'])) {
        $actions = is_array($transition['action']) ? $transition['action'] : [$transition['action']];
        foreach ($actions as $actionId) {
          $plugin = $this->action->createInstance($actionId);
          if (!empty($plugin) && method_exists($plugin, 'execute')) {
            $plugin->execute($entity, $transition, ['transitions' => $this->transitions]);
          }
        }
      }
    }
    return $this;
  }

  /**
   * Check transition is allowed to execute.
   *
   * {@inheritdoc}
   */
  public function isTransitionAllowed($entity = NULL) {
    $transition = $this->getTransition();
    // Check guard with id plugin guard.
    if (!empty($transition['guard'])) {
      $plugin = $this->guard->createInstance($transition['guard']);
      if (!empty($plugin) && method_exists($plugin, 'allowed')) {
        return $plugin->allowed($transition, ['transitions' => $this->getTransitions()], $entity);
      }
    }
    // Check permission.
    if (!empty($transition['permission'])) {
      return $this->currentUser->hasPermission($transition['permission']);
    }
    // Check role.
    $userRoles = $this->currentUser->getRoles();
    if (in_array('administrator', $userRoles)) {
      return TRUE;
    }
    if (!empty($transition['role'])) {
      $workflowRoles = is_array($transition['role']) ? $transition['role'] : [$transition['role']];
      $intersection = array_intersect($workflowRoles, $userRoles);
      return !empty($intersection);
    }
    return FALSE;
  }

  /**
   * Set transitions by field.
   *
   * {@inheritdoc}
   */
  public function setTransitionByField($entity, $field_name) {
    $fieldDefinition = $entity->getFieldDefinition($field_name);
    $workflow = trim($fieldDefinition->getSetting('workflows'));
    if (!empty($workflow)) {
      $workflow = Yaml::decode($workflow);
      if (!empty($workflow['transitions'])) {
        $this->transitions = $workflow['transitions'];
      }
    }
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getPossibleTransitions($state) {
    $possible_transitions = [];
    foreach ($this->transitions as $id => $transition) {
      if (in_array($state, $transition['from'])) {
        $possible_transitions[$id] = $transition;
      }
    }
    return $this->setTransitions($possible_transitions);
  }

}
